@@ -2,6 +2,7 @@ module Debian.Deb where
 
 import Control.Monad
 
+import Data.ByteString.Lazy (empty)
 import Debian.Control.Common
 import System.Unix.Directory
 import System.Unix.FilePath
@@ -12,9 +13,9 @@ fields debFP =
     withTemporaryDirectory ("fields.XXXXXX") $ \tmpdir ->
       do debFP <- realpath debFP
          withWorkingDirectory tmpdir $ 
-           do (out, err, res) <- simpleProcess "ar" ["x",debFP,"control.tar.gz"] 
+           do (out, err, res) <- lazyProcess "ar" ["x",debFP,"control.tar.gz"] Nothing Nothing empty >>= return . collectOutputUnpacked 
               when (res /= ExitSuccess) (error $ "Dpkg.fields: " ++ out ++ "\n" ++ err ++ "\n" ++ show res)
-              (out, err, res) <- simpleProcess "tar" ["xzf", "control.tar.gz", "./control"]
+              (out, err, res) <- lazyProcess "tar" ["xzf", "control.tar.gz", "./control"] Nothing Nothing empty >>= return . collectOutputUnpacked
               when (res /= ExitSuccess) (error $ "Dpkg.fields: " ++ out ++ "\n" ++ err ++ "\n" ++ show res)
               c <- parseControlFromFile "control" 
               case c of
